@@ -29,10 +29,11 @@ from .parameterized import (
     Parameterized, Parameter, String, ParameterizedFunction, ParamOverrides,
     descendents, get_logger, instance_descriptor, basestring)
 
-from .parameterized import (batch_watch, depends, output, # noqa: api import
+from .parameterized import (batch_watch, depends, output, script_repr, # noqa: api import
                             discard_events, edit_constant, instance_descriptor)
-from .parameterized import logging_level     # noqa: api import
 from .parameterized import shared_parameters # noqa: api import
+from .parameterized import logging_level     # noqa: api import
+from .parameterized import DEBUG, VERBOSE, INFO, WARNING, ERROR, CRITICAL # noqa: api import
 
 from collections import OrderedDict
 from numbers import Real
@@ -42,7 +43,7 @@ from numbers import Real
 # only two required files.
 try:
     from .version import Version
-    __version__ = str(Version(fpath=__file__, archive_commit="7752ddd9", reponame="param"))
+    __version__ = str(Version(fpath=__file__, archive_commit="d490aa95", reponame="param"))
 except:
     __version__ = "0.0.0+unknown"
 
@@ -70,7 +71,7 @@ main=Parameterized(name="main")
 
 
 # A global random seed (integer or rational) available for controlling
-# the behaviour of parameterized objects with random state.
+# the behaviour of Parameterized objects with random state.
 random_seed = 42
 
 
@@ -469,7 +470,7 @@ class Time(Parameterized):
             raise StopIteration
         return self._time
 
-    # For Python 2 compatibility; can be removed for Python 3.
+    # PARAM2_DEPRECATION: For Python 2 compatibility; can be removed for Python 3.
     next = __next__
 
     def __call__(self, val=None, time_type=None):
@@ -565,10 +566,6 @@ class Dynamic(Parameter):
     time_fn = Time()
     time_dependent = False
 
-    # CBENHANCEMENT: Add an 'epsilon' slot.
-    # See email 'Re: simulation-time-controlled Dynamic parameters'
-    # Dec 22, 2007 CB->JAB
-
     def __init__(self,**params):
         """
         Call the superclass's __init__ and set instantiate=True if the
@@ -585,12 +582,12 @@ class Dynamic(Parameter):
         """
         Add 'last time' and 'last value' attributes to the generator.
         """
-        # CEBALERT: use a dictionary to hold these things.
+        # Could use a dictionary to hold these things.
         if hasattr(obj,"_Dynamic_time_fn"):
             gen._Dynamic_time_fn = obj._Dynamic_time_fn
 
         gen._Dynamic_last = None
-        # CEB: I'd use None for this, except can't compare a fixedpoint
+        # Would have usede None for this, but can't compare a fixedpoint
         # number with None (e.g. 1>None but FixedPoint(1)>None can't be done)
         gen._Dynamic_time = -1
 
@@ -803,9 +800,9 @@ class Number(Dynamic):
         dynamically generated, check the bounds.
         """
         result = super(Number, self).__get__(obj, objtype)
-        # CEBALERT: results in extra lookups (_value_is_dynamic() is
-        # also looking up 'result' - should just pass it in). Note
-        # that this method is called often.
+        # Should be able to optimize this commonly used method by
+        # avoiding extra lookups (e.g. _value_is_dynamic() is also
+        # looking up 'result' - should just pass it in).
         if self._value_is_dynamic(obj, objtype):
             self._validate(result)
         return result
@@ -822,8 +819,6 @@ class Number(Dynamic):
             bounded_val = val
         super(Number, self).__set__(obj, bounded_val)
 
-    # CEBERRORALERT: doesn't take account of exclusive bounds; see
-    # https://github.com/ioam/param/issues/80.
     def crop_to_bounds(self, val):
         """
         Return the given value cropped to be within the hard bounds
@@ -835,10 +830,14 @@ class Number(Dynamic):
         returned value could be None.  If a non-numeric value is passed
         in, set to be the default value (which could be None).  In no
         case is an exception raised; all values are accepted.
+
+        As documented in https://github.com/holoviz/param/issues/80,
+        currently does not respect exclusive bounds, which would
+        strictly require setting to one less for integer values or
+        an epsilon less for floats.
         """
-        # Currently, values outside the bounds are silently cropped to
-        # be inside the bounds; it may be appropriate to add a warning
-        # in such cases.
+        # Values outside the bounds are silently cropped to
+        # be inside the bounds.
         if _is_number(val):
             if self.bounds is None:
                 return val
@@ -955,7 +954,8 @@ class Boolean(Parameter):
 
     __slots__ = ['bounds']
 
-    # CB: bounds have no effect; see https://github.com/holoviz/param/issues/82
+    # Bounds are set for consistency and are arguably accurate, but have
+    # no effect since values are either False, True, or None (if allowed).
     def __init__(self, default=False, bounds=(0,1), **params):
         self.bounds = bounds
         super(Boolean, self).__init__(default=default, **params)
@@ -1078,7 +1078,7 @@ def _is_abstract(class_):
         return False
 
 
-# CEBALERT: this should be a method of ClassSelector.
+# Could be a method of ClassSelector.
 def concrete_descendents(parentclass):
     """
     Return a dictionary containing all subclasses of the specified
@@ -1161,9 +1161,9 @@ class Selector(SelectorBase):
     """
     Parameter whose value must be one object from a list of possible objects.
 
-    By default, if no default is specfied, picks the first object from
-    the provided set of objects, as long as the objects are in an ordered
-    data collection.
+    By default, if no default is specified, picks the first object from
+    the provided set of objects, as long as the objects are in an
+    ordered data collection.
 
     check_on_set restricts the value to be among the current list of
     objects. By default, if objects are initially supplied,
@@ -1231,8 +1231,8 @@ class Selector(SelectorBase):
         if default is not None and self.check_on_set is True:
             self._validate(default)
 
-    # CBNOTE: if the list of objects is changed, the current value for
-    # this parameter in existing POs could be out of the new range.
+    # Note that if the list of objects is changed, the current value for
+    # this parameter in existing POs could be outside of the new range.
 
     def compute_default(self):
         """
@@ -1349,8 +1349,8 @@ class ClassSelector(SelectorBase):
         """
         Return the possible types for this parameter's value.
 
-        (I.e. return {name: <class>} for all classes that are
-        concrete_descendents() of self.class_.)
+        (I.e. return `{name: <class>}` for all classes that are
+        concrete_descendents() of `self.class_`.)
 
         Only classes from modules that have been imported are added
         (see concrete_descendents()).
@@ -1609,8 +1609,8 @@ class Series(ClassSelector):
 
 # For portable code:
 #   - specify paths in unix (rather than Windows) style;
-#   - use resolve_file_path() for paths to existing files to be read,
-#   - use resolve_folder_path() for paths to existing folders to be read,
+#   - use resolve_path(path_to_file=True) for paths to existing files to be read,
+#   - use resolve_path(path_to_file=False) for paths to existing folders to be read,
 #     and normalize_path() for paths to new files to be written.
 
 class resolve_path(ParameterizedFunction):
@@ -1636,7 +1636,9 @@ class resolve_path(ParameterizedFunction):
 
     path_to_file = Boolean(default=True, pickle_default_value=False,
                            allow_None=True, doc="""
-        String specifying whether the path refers to a 'File' or a 'Folder'.""")
+        String specifying whether the path refers to a 'File' or a
+        'Folder'. If None, the path may point to *either* a 'File' *or*
+        a 'Folder'.""")
 
     def __call__(self, path, **params):
         p = ParamOverrides(self, params)
@@ -1707,7 +1709,7 @@ class Path(Parameter):
     The specified path can be absolute, or relative to either:
 
     * any of the paths specified in the search_paths attribute (if
-      search_paths is not None);
+       search_paths is not None);
 
     or
 
@@ -1766,6 +1768,7 @@ class Filename(Path):
 
     * any of the paths specified in the search_paths attribute (if
       search_paths is not None);
+
     or
 
     * any of the paths searched by resolve_path() (if search_paths
@@ -1787,6 +1790,7 @@ class Foldername(Path):
 
     * any of the paths specified in the search_paths attribute (if
       search_paths is not None);
+
     or
 
     * any of the paths searched by resolve_dir_path() (if search_paths
@@ -1856,6 +1860,8 @@ class ListSelector(Selector):
                     self.objects.append(o)
 
     def _validate(self, val):
+        if (val is None and self.allow_None):
+            return
         for o in val:
             super(ListSelector, self)._validate(o)
 
@@ -1938,8 +1944,8 @@ class CalendarDate(Number):
         if self.allow_None and val is None:
             return
 
-        if not isinstance(val, dt.date) and not (allow_None and val is None):
-            raise ValueError("CalendarDate parameter %r only takes datetime types." % self.name)
+        if (not isinstance(val, dt.date) or isinstance(val, dt.datetime)) and not (allow_None and val is None):
+            raise ValueError("CalendarDate parameter %r only takes date types." % self.name)
 
     def _validate_step(self, val, step):
         if step is not None and not isinstance(step, dt.date):
@@ -2029,7 +2035,9 @@ class Color(Parameter):
 
 
 class Range(NumericTuple):
-    "A numeric range with optional bounds and softbounds"
+    """
+    A numeric range with optional bounds and softbounds.
+    """
 
     __slots__ = ['bounds', 'inclusive_bounds', 'softbounds', 'step']
 
